﻿
    using System.ComponentModel;
    using System.Globalization;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Controls.Primitives;
    using System.Windows.Input;
    using System.Windows.Media;
    using Syncfusion.Windows.Controls.Cells;
    using Syncfusion.Windows.Controls.Grid;
    using Syncfusion.Windows.Controls.Scroll;
    using Syncfusion.Windows.GridCommon;
    using System;

namespace Dropdownlist
{
    public class CustomGridCellModel : GridCellModel<CustomGridCellRenderer>
    {
        public CustomGridCellModel()
        {
                
        }
    }

   public class CustomGridCellRenderer : GridVirtualizingCellRenderer<ContentControl>
    {
       public CustomGridCellRenderer()
       {
           SupportsRenderOptimization = true;
           this.IsFocusable = true;
           AllowRecycle = true;
           AllowKeepAliveOnlyCurrentCell = true;
       }

        public override void OnInitializeContent(ContentControl uiElement, GridRenderStyleInfo style)
        {

            bool found = false;
            if (style.CellEditTemplateKey != null)
            {
                DataTemplate dt = (DataTemplate)style.GridControl.TryFindResource(style.CellEditTemplateKey);
                found = dt != null;

                if (found)
                {
                    var wrapperInstance = uiElement.DataContext as TemplateCellBoundWrapper;
                    if (wrapperInstance == null)
                    {
                        wrapperInstance = new TemplateCellBoundWrapper() { CellBoundValue = this.ControlValue };
                        wrapperInstance.ValueChanged += wrapperInstance_ValueChanged;
                    }
                    if (wrapperInstance.CellBoundValue != this.ControlValue)
                        wrapperInstance.CellBoundValue = this.ControlValue;

                    uiElement.Content = wrapperInstance;
                    uiElement.SetValue(ContentControl.DataContextProperty, wrapperInstance);
                    uiElement.ContentTemplate = dt;
                }
            }
            GridControlBase.SetDelayLoad(uiElement, false);

            base.OnInitializeContent(uiElement, style);
        }

        protected override void ArrangeUIElement(ArrangeCellArgs aca, ContentControl uiElement, GridRenderStyleInfo style)
        {
            bool found = false;
            if (style.CellEditTemplateKey != null)
            {
                DataTemplate dt = (DataTemplate)style.GridControl.TryFindResource(style.CellEditTemplateKey);
                found = dt != null;

                if (found)
                {
                    var wrapperInstance = uiElement.DataContext as TemplateCellBoundWrapper;
                    if (wrapperInstance == null)
                        wrapperInstance = new TemplateCellBoundWrapper() { CellBoundValue = this.ControlValue };
                    if (wrapperInstance.CellBoundValue != this.ControlValue)
                        wrapperInstance.CellBoundValue = this.ControlValue;

                    uiElement.Content = wrapperInstance;
                    uiElement.SetValue(ContentControl.DataContextProperty, wrapperInstance);
                    uiElement.ContentTemplate = dt;
                }
            }
            base.ArrangeUIElement(aca, uiElement, style);
        }

        public virtual void wrapperInstance_ValueChanged(object sender, GridDataValueEventArgs<object> e)
        {
            if (!this.IsInArrange && !this.CurrentCell.IsInEndEdit)
            {
                if (!this.SetControlValue(e.Value))
                {
                    RefreshContent();
                }
                this.CurrentStyle.CellValue = e.Value;
            }
        }

        protected override void OnRender(DrawingContext dc, RenderCellArgs rca, GridRenderStyleInfo style)
        {
            if (rca.CellUIElements != null)
                return;

            // Will only get hit if SupportsRenderOptimization is true, otherwise rca.CellUIElements is never null.
            Thickness margins = style.TextMargins.ToThickness();
            Rect textRectangle = rca.SubtractBorderMargins(rca.CellRect, margins);
            margins.Left = Math.Max(margins.Left, 2);
            margins.Right = Math.Max(margins.Right, 2);

            textRectangle = rca.SubtractBorderMargins(rca.CellRect, margins);

            if (textRectangle.IsEmpty)
                return;

            string text;
            if (IsCurrentCell(style) && HasControlText && this.CurrentCell.IsEditing)
                text = ControlText;
            else
                text = GetControlText(style);

            double dValue = 0;

            if (style.CellValue != null && double.TryParse(style.CellValue.ToString(), out dValue))
            {
                if (dValue < 0)
                {
                    style.Foreground = style.HasNegativeForeground ? style.NegativeForeground : style.Foreground;
                }
            }

            if (text.Length != 0 && text.Length > style.MaxLength && style.HasMaxLength)
                text = text.Remove(style.MaxLength);
            
            // Draw the formatted text string to the DrawingContext of the control.
            GridTextBoxPaint.DrawText(dc, textRectangle, text, style);
        }

    }

   public class TemplateCellBoundWrapper : DependencyObject
   {
       public static readonly DependencyProperty CellBoundValueProperty = DependencyProperty.Register("CellBoundValue", typeof(object), typeof(TemplateCellBoundWrapper), new FrameworkPropertyMetadata(OnValueChanged));

       public object CellBoundValue
       {
           get
           {
               return this.GetValue(TemplateCellBoundWrapper.CellBoundValueProperty);
           }
           set
           {
               this.SetValue(TemplateCellBoundWrapper.CellBoundValueProperty, value);
           }
       }

       
       private static void OnValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs args)
       {
           var boundObject = d as TemplateCellBoundWrapper;
           if (boundObject != null)
           {
               boundObject.RaiseOnValueChanged(args.NewValue);
           }
       }

       private void RaiseOnValueChanged(object value)
       {
           if (this.ValueChanged != null)
           {
               this.ValueChanged(this, new GridDataValueEventArgs<object>(value));
           }
       }

       /// <summary>
       /// Occurs when the cell bound value changes.
       /// </summary>
       public event EventHandler<GridDataValueEventArgs<object>> ValueChanged;
   }
}
